@@ -50,7 +50,7 @@ function Buffer(subject, encoding) {
     return new Buffer(subject, encoding);
 
   if (util.isNumber(subject)) {
-    this.length = subject > 0 ? subject >>> 0 : 0;
+    this.length = +subject;
 
   } else if (util.isString(subject)) {
     if (!util.isString(encoding) || encoding.length === 0)
@@ -61,8 +61,7 @@ function Buffer(subject, encoding) {
   } else if (util.isObject(subject)) {
     if (subject.type === 'Buffer' && util.isArray(subject.data))
       subject = subject.data;
-    // Must use floor() because array length may be > kMaxLength.
-    this.length = +subject.length > 0 ? Math.floor(+subject.length) : 0;
+    this.length = +subject.length;
 
   } else {
     throw new TypeError('must start with number, buffer, array or string');
@@ -73,6 +72,11 @@ function Buffer(subject, encoding) {
                          'size: 0x' + kMaxLength.toString(16) + ' bytes');
   }
 
+  if (this.length < 0)
+    this.length = 0;
+  else
+    this.length >>>= 0;  // Coerce to uint32.
+
   this.parent = undefined;
   if (this.length <= (Buffer.poolSize >>> 1) && this.length > 0) {
     if (this.length > poolSize - poolOffset)
@@ -82,6 +86,12 @@ function Buffer(subject, encoding) {
                             poolOffset,
                             poolOffset + this.length);
     poolOffset += this.length;
+
+    // Ensure aligned slices
+    if (poolOffset & 0x7) {
+      poolOffset |= 0x7;
+      poolOffset++;
+    }
   } else {
     alloc(this, this.length);
   }
@@ -100,7 +110,9 @@ function Buffer(subject, encoding) {
       var prevLen = this.length;
       this.length = len;
       truncate(this, this.length);
-      poolOffset -= (prevLen - len);
+      // Only need to readjust the poolOffset if the allocation is a slice.
+      if (this.parent != undefined)
+        poolOffset -= (prevLen - len);
     }
 
   } else if (util.isBuffer(subject)) {
@@ -181,7 +193,7 @@ Buffer.isEncoding = function(encoding) {
 
 Buffer.concat = function(list, length) {
   if (!util.isArray(list))
-    throw new TypeError('Usage: Buffer.concat(list[, length])');
+    throw new TypeError('list argument must be an Array of Buffers.');
 
   if (util.isUndefined(length)) {
     length = 0;
