@@ -313,6 +313,9 @@ function getSocketList(type, slave, key) {
 
 var INTERNAL_PREFIX = 'NODE_';
 function handleMessage(target, message, handle) {
+  if (!target._channel)
+    return;
+
   var eventName = 'message';
   if (!util.isNull(message) &&
       util.isObject(message) &&
@@ -967,6 +970,7 @@ function normalizeSpawnArguments(file /*, args, options*/) {
   else if (!util.isObject(options))
     throw new TypeError('options argument must be an object');
 
+  options = util._extend({}, options);
   args.unshift(file);
 
   var env = options.env || process.env;
@@ -1093,8 +1097,11 @@ util.inherits(ChildProcess, EventEmitter);
 function flushStdio(subprocess) {
   if (subprocess.stdio == null) return;
   subprocess.stdio.forEach(function(stream, fd, stdio) {
-    if (!stream || !stream.readable || stream._consuming ||
-        stream._readableState.flowing)
+    if (!stream ||
+        !stream.readable ||
+        stream._consuming ||
+        stream._readableState.flowing ||
+        stream._readableState.readableListening)
       return;
     stream.resume();
   });
