@@ -26,9 +26,9 @@
 // * http://www.3waylabs.com/nw/WWW/products/wizcon/vt220.html
 
 var kHistorySize = 30;
-var kBufSize = 10 * 1024;
 
 var util = require('util');
+var Buffer = require('buffer').Buffer;
 var inherits = require('util').inherits;
 var EventEmitter = require('events').EventEmitter;
 
@@ -63,13 +63,13 @@ function Interface(input, output, completer, terminal) {
 
   completer = completer || function() { return []; };
 
-  if (typeof completer !== 'function') {
+  if (!util.isFunction(completer)) {
     throw new TypeError('Argument \'completer\' must be a function');
   }
 
   // backwards compat; check the isTTY prop of the output stream
   //  when `terminal` was not specified
-  if (typeof terminal == 'undefined') {
+  if (util.isUndefined(terminal) && !util.isNullOrUndefined(output)) {
     terminal = !!output.isTTY;
   }
 
@@ -92,6 +92,16 @@ function Interface(input, output, completer, terminal) {
   }
 
   function onend() {
+    if (util.isString(self._line_buffer) && self._line_buffer.length > 0) {
+      self.emit('line', self._line_buffer);
+    }
+    self.close();
+  }
+
+  function ontermend() {
+    if (util.isString(self.line) && self.line.length > 0) {
+      self.emit('line', self.line);
+    }
     self.close();
   }
 
@@ -119,6 +129,7 @@ function Interface(input, output, completer, terminal) {
 
     // input usually refers to stdin
     input.on('keypress', onkeypress);
+    input.on('end', ontermend);
 
     // Current line
     this.line = '';
@@ -132,10 +143,15 @@ function Interface(input, output, completer, terminal) {
     this.history = [];
     this.historyIndex = -1;
 
-    output.on('resize', onresize);
+    if (!util.isNullOrUndefined(output))
+      output.on('resize', onresize);
+
     self.once('close', function() {
       input.removeListener('keypress', onkeypress);
-      output.removeListener('resize', onresize);
+      input.removeListener('end', ontermend);
+      if (!util.isNullOrUndefined(output)) {
+        output.removeListener('resize', onresize);
+      }
     });
   }
 
@@ -145,23 +161,19 @@ function Interface(input, output, completer, terminal) {
 inherits(Interface, EventEmitter);
 
 Interface.prototype.__defineGetter__('columns', function() {
-  return this.output.columns || Infinity;
+  var columns = Infinity;
+  if (this.output && this.output.columns)
+    columns = this.output.columns;
+  return columns;
 });
 
-Interface.prototype.setPrompt = function(prompt, length) {
+Interface.prototype.setPrompt = function(prompt) {
   this._prompt = prompt;
-  if (length) {
-    this._promptLength = length;
-  } else {
-    var lines = prompt.split(/[\r\n]/);
-    var lastLine = lines[lines.length - 1];
-    this._promptLength = lastLine.length;
-  }
 };
 
 
 Interface.prototype._setRawMode = function(mode) {
-  if (typeof this.input.setRawMode === 'function') {
+  if (util.isFunction(this.input.setRawMode)) {
     return this.input.setRawMode(mode);
   }
 };
@@ -173,13 +185,13 @@ Interface.prototype.prompt = function(preserveCursor) {
     if (!preserveCursor) this.cursor = 0;
     this._refreshLine();
   } else {
-    this.output.write(this._prompt);
+    this._writeToOutput(this._prompt);
   }
 };
 
 
 Interface.prototype.question = function(query, cb) {
-  if (typeof cb === 'function') {
+  if (util.isFunction(cb)) {
     if (this._questionCallback) {
       this.prompt();
     } else {
@@ -203,6 +215,13 @@ Interface.prototype._onLine = function(line) {
   }
 };
 
+Interface.prototype._writeToOutput = function _writeToOutput(stringToWrite) {
+  if (!util.isString(stringToWrite))
+    throw new TypeError('stringToWrite must be a string');
+
+  if (!util.isNullOrUndefined(this.output))
+    this.output.write(stringToWrite);
+};
 
 Interface.prototype._addHistory = function() {
   if (this.line.length === 0) return '';
@@ -220,13 +239,11 @@ Interface.prototype._addHistory = function() {
 
 
 Interface.prototype._refreshLine = function() {
-  var columns = this.columns;
-
   // line length
   var line = this._prompt + this.line;
-  var lineLength = line.length;
-  var lineCols = lineLength % columns;
-  var lineRows = (lineLength - lineCols) / columns;
+  var dispPos = this._getDisplayPos(line);
+  var lineCols = dispPos.cols;
+  var lineRows = dispPos.rows;
 
   // cursor position
   var cursorPos = this._getCursorPos();
@@ -243,11 +260,11 @@ Interface.prototype._refreshLine = function() {
   exports.clearScreenDown(this.output);
 
   // Write the prompt and the current buffer content.
-  this.output.write(line);
+  this._writeToOutput(line);
 
   // Force terminal to allocate a new line
   if (lineCols === 0) {
-    this.output.write(' ');
+    this._writeToOutput(' ');
   }
 
   // Move cursor to original position.
@@ -264,10 +281,10 @@ Interface.prototype._refreshLine = function() {
 
 Interface.prototype.close = function() {
   if (this.closed) return;
+  this.pause();
   if (this.terminal) {
     this._setRawMode(false);
   }
-  this.pause();
   this.closed = true;
   this.emit('close');
 };
@@ -278,6 +295,7 @@ Interface.prototype.pause = function() {
   this.input.pause();
   this.paused = true;
   this.emit('pause');
+  return this;
 };
 
 
@@ -286,6 +304,7 @@ Interface.prototype.resume = function() {
   this.input.resume();
   this.paused = false;
   this.emit('resume');
+  return this;
 };
 
 
@@ -297,7 +316,7 @@ Interface.prototype.write = function(d, key) {
 // \r\n, \n, or \r followed by something other than \n
 var lineEnding = /\r?\n|\r(?!\n)/;
 Interface.prototype._normalWrite = function(b) {
-  if (b === undefined) {
+  if (util.isUndefined(b)) {
     return;
   }
   var string = this._decoder.write(b);
@@ -306,11 +325,14 @@ Interface.prototype._normalWrite = function(b) {
     this._sawReturn = false;
   }
 
+  // Run test() on the new string chunk, not on the entire line buffer.
+  var newPartContainsEnding = lineEnding.test(string);
+
   if (this._line_buffer) {
     string = this._line_buffer + string;
     this._line_buffer = null;
   }
-  if (lineEnding.test(string)) {
+  if (newPartContainsEnding) {
     this._sawReturn = /\r$/.test(string);
 
     // got one or more newlines; process into "line" events
@@ -344,7 +366,7 @@ Interface.prototype._insertString = function(c) {
     if (this._getCursorPos().cols === 0) {
       this._refreshLine();
     } else {
-      this.output.write(c);
+      this._writeToOutput(c);
     }
 
     // a hack to get the line refreshed if it's needed
@@ -371,7 +393,7 @@ Interface.prototype._tabComplete = function() {
       if (completions.length === 1) {
         self._insertString(completions[0].slice(completeOn.length));
       } else {
-        self.output.write('\r\n');
+        self._writeToOutput('\r\n');
         var width = completions.reduce(function(a, b) {
           return a.length > b.length ? a : b;
         }).length + 2;  // 2 space padding
@@ -415,17 +437,17 @@ function handleGroup(self, group, width, maxColumns) {
         break;
       }
       var item = group[idx];
-      self.output.write(item);
+      self._writeToOutput(item);
       if (col < maxColumns - 1) {
         for (var s = 0, itemLen = item.length; s < width - itemLen;
              s++) {
-          self.output.write(' ');
+          self._writeToOutput(' ');
         }
       }
     }
-    self.output.write('\r\n');
+    self._writeToOutput('\r\n');
   }
-  self.output.write('\r\n');
+  self._writeToOutput('\r\n');
 }
 
 function commonPrefix(strings) {
@@ -518,7 +540,7 @@ Interface.prototype._deleteLineRight = function() {
 
 Interface.prototype.clearLine = function() {
   this._moveCursor(+Infinity);
-  this.output.write('\r\n');
+  this._writeToOutput('\r\n');
   this.line = '';
   this.cursor = 0;
   this.prevRows = 0;
@@ -559,12 +581,53 @@ Interface.prototype._historyPrev = function() {
 };
 
 
+// Returns the last character's display position of the given string
+Interface.prototype._getDisplayPos = function(str) {
+  var offset = 0;
+  var col = this.columns;
+  var row = 0;
+  var code;
+  str = stripVTControlCharacters(str);
+  for (var i = 0, len = str.length; i < len; i++) {
+    code = codePointAt(str, i);
+    if (code >= 0x10000) { // surrogates
+      i++;
+    }
+    if (code === 0x0a) { // new line \n
+      offset = 0;
+      row += 1;
+      continue;
+    }
+    if (isFullWidthCodePoint(code)) {
+      if ((offset + 1) % col === 0) {
+        offset++;
+      }
+      offset += 2;
+    } else {
+      offset++;
+    }
+  }
+  var cols = offset % col;
+  var rows = row + (offset - cols) / col;
+  return {cols: cols, rows: rows};
+};
+
+
 // Returns current cursor's position and line
 Interface.prototype._getCursorPos = function() {
   var columns = this.columns;
-  var cursorPos = this.cursor + this._promptLength;
-  var cols = cursorPos % columns;
-  var rows = (cursorPos - cols) / columns;
+  var strBeforeCursor = this._prompt + this.line.substring(0, this.cursor);
+  var dispPos = this._getDisplayPos(stripVTControlCharacters(strBeforeCursor));
+  var cols = dispPos.cols;
+  var rows = dispPos.rows;
+  // If the cursor is on a full-width character which steps over the line,
+  // move the cursor to the beginning of the next line.
+  if (cols + 1 === columns &&
+      this.cursor < this.line.length &&
+      isFullWidthCodePoint(codePointAt(this.line, this.cursor))) {
+    rows++;
+    cols = 0;
+  }
   return {cols: cols, rows: rows};
 };
 
@@ -578,13 +641,24 @@ Interface.prototype._moveCursor = function(dx) {
 
   // bounds check
   if (this.cursor < 0) this.cursor = 0;
-  if (this.cursor > this.line.length) this.cursor = this.line.length;
+  else if (this.cursor > this.line.length) this.cursor = this.line.length;
 
   var newPos = this._getCursorPos();
 
   // check if cursors are in the same line
   if (oldPos.rows === newPos.rows) {
-    exports.moveCursor(this.output, this.cursor - oldcursor, 0);
+    var diffCursor = this.cursor - oldcursor;
+    var diffWidth;
+    if (diffCursor < 0) {
+      diffWidth = -getStringWidth(
+          this.line.substring(this.cursor, oldcursor)
+          );
+    } else if (diffCursor > 0) {
+      diffWidth = getStringWidth(
+          this.line.substring(this.cursor, oldcursor)
+          );
+    }
+    exports.moveCursor(this.output, diffWidth, 0);
     this.prevRows = newPos.rows;
   } else {
     this._refreshLine();
@@ -663,6 +737,12 @@ Interface.prototype._ttyWrite = function(s, key) {
         this._moveCursor(+1);
         break;
 
+      case 'l': // clear the whole screen
+        exports.cursorTo(this.output, 0, 0);
+        exports.clearScreenDown(this.output);
+        this._refreshLine();
+        break;
+
       case 'n': // next history item
         this._historyNext();
         break;
@@ -685,7 +765,7 @@ Interface.prototype._ttyWrite = function(s, key) {
                 self.pause();
                 self.emit('SIGCONT');
               }
-              // explictly re-enable "raw mode" and move the cursor to
+              // explicitly re-enable "raw mode" and move the cursor to
               // the correct position.
               // See https://github.com/joyent/node/issues/3295.
               self._setRawMode(true);
@@ -706,10 +786,6 @@ Interface.prototype._ttyWrite = function(s, key) {
         this._deleteWordRight();
         break;
 
-      case 'backspace':
-        this._deleteWordLeft();
-        break;
-
       case 'left':
         this._wordLeft();
         break;
@@ -798,7 +874,7 @@ Interface.prototype._ttyWrite = function(s, key) {
         break;
 
       default:
-        if (Buffer.isBuffer(s))
+        if (util.isBuffer(s))
           s = s.toString('utf-8');
 
         if (s) {
@@ -831,7 +907,7 @@ function emitKeypressEvents(stream) {
   function onData(b) {
     if (EventEmitter.listenerCount(stream, 'keypress') > 0) {
       var r = stream._keypressDecoder.write(b);
-      if (r) emitKey(stream, r);
+      if (r) emitKeys(stream, r);
     } else {
       // Nobody's watching anyway
       stream.removeListener('data', onData);
@@ -883,22 +959,21 @@ exports.emitKeypressEvents = emitKeypressEvents;
 */
 
 // Regexes used for ansi escape code splitting
-var metaKeyCodeRe = /^(?:\x1b)([a-zA-Z0-9])$/;
-var functionKeyCodeRe =
-    /^(?:\x1b+)(O|N|\[|\[\[)(?:(\d+)(?:;(\d+))?([~^$])|(?:1;)?(\d+)?([a-zA-Z]))/;
+var metaKeyCodeReAnywhere = /(?:\x1b)([a-zA-Z0-9])/;
+var metaKeyCodeRe = new RegExp('^' + metaKeyCodeReAnywhere.source + '$');
+var functionKeyCodeReAnywhere = new RegExp('(?:\x1b+)(O|N|\\[|\\[\\[)(?:' + [
+  '(\\d+)(?:;(\\d+))?([~^$])',
+  '(?:M([@ #!a`])(.)(.))', // mouse
+  '(?:1;)?(\\d+)?([a-zA-Z])'
+].join('|') + ')');
+var functionKeyCodeRe = new RegExp('^' + functionKeyCodeReAnywhere.source);
+var escapeCodeReAnywhere = new RegExp([
+  functionKeyCodeReAnywhere.source, metaKeyCodeReAnywhere.source, /\x1b./.source
+].join('|'));
 
-function emitKey(stream, s) {
-  var ch,
-      key = {
-        name: undefined,
-        ctrl: false,
-        meta: false,
-        shift: false
-      },
-      parts;
-
-  if (Buffer.isBuffer(s)) {
-    if (s[0] > 127 && s[1] === undefined) {
+function emitKeys(stream, s) {
+  if (util.isBuffer(s)) {
+    if (s[0] > 127 && util.isUndefined(s[1])) {
       s[0] -= 128;
       s = '\x1b' + s.toString(stream.encoding || 'utf-8');
     } else {
@@ -906,188 +981,200 @@ function emitKey(stream, s) {
     }
   }
 
-  key.sequence = s;
+  var buffer = [];
+  var match;
+  while (match = escapeCodeReAnywhere.exec(s)) {
+    buffer = buffer.concat(s.slice(0, match.index).split(''));
+    buffer.push(match[0]);
+    s = s.slice(match.index + match[0].length);
+  }
+  buffer = buffer.concat(s.split(''));
 
-  if (s === '\r') {
-    // carriage return
-    key.name = 'return';
+  buffer.forEach(function(s) {
+    var ch,
+        key = {
+          sequence: s,
+          name: undefined,
+          ctrl: false,
+          meta: false,
+          shift: false
+        },
+        parts;
 
-  } else if (s === '\n') {
-    // enter, should have been called linefeed
-    key.name = 'enter';
+    if (s === '\r') {
+      // carriage return
+      key.name = 'return';
 
-  } else if (s === '\t') {
-    // tab
-    key.name = 'tab';
+    } else if (s === '\n') {
+      // enter, should have been called linefeed
+      key.name = 'enter';
 
-  } else if (s === '\b' || s === '\x7f' ||
-             s === '\x1b\x7f' || s === '\x1b\b') {
-    // backspace or ctrl+h
-    key.name = 'backspace';
-    key.meta = (s.charAt(0) === '\x1b');
+    } else if (s === '\t') {
+      // tab
+      key.name = 'tab';
 
-  } else if (s === '\x1b' || s === '\x1b\x1b') {
-    // escape key
-    key.name = 'escape';
-    key.meta = (s.length === 2);
+    } else if (s === '\b' || s === '\x7f' ||
+               s === '\x1b\x7f' || s === '\x1b\b') {
+      // backspace or ctrl+h
+      key.name = 'backspace';
+      key.meta = (s.charAt(0) === '\x1b');
 
-  } else if (s === ' ' || s === '\x1b ') {
-    key.name = 'space';
-    key.meta = (s.length === 2);
+    } else if (s === '\x1b' || s === '\x1b\x1b') {
+      // escape key
+      key.name = 'escape';
+      key.meta = (s.length === 2);
 
-  } else if (s <= '\x1a') {
-    // ctrl+letter
-    key.name = String.fromCharCode(s.charCodeAt(0) + 'a'.charCodeAt(0) - 1);
-    key.ctrl = true;
+    } else if (s === ' ' || s === '\x1b ') {
+      key.name = 'space';
+      key.meta = (s.length === 2);
 
-  } else if (s.length === 1 && s >= 'a' && s <= 'z') {
-    // lowercase letter
-    key.name = s;
+    } else if (s.length === 1 && s <= '\x1a') {
+      // ctrl+letter
+      key.name = String.fromCharCode(s.charCodeAt(0) + 'a'.charCodeAt(0) - 1);
+      key.ctrl = true;
 
-  } else if (s.length === 1 && s >= 'A' && s <= 'Z') {
-    // shift+letter
-    key.name = s.toLowerCase();
-    key.shift = true;
+    } else if (s.length === 1 && s >= 'a' && s <= 'z') {
+      // lowercase letter
+      key.name = s;
 
-  } else if (parts = metaKeyCodeRe.exec(s)) {
-    // meta+character key
-    key.name = parts[1].toLowerCase();
-    key.meta = true;
-    key.shift = /^[A-Z]$/.test(parts[1]);
+    } else if (s.length === 1 && s >= 'A' && s <= 'Z') {
+      // shift+letter
+      key.name = s.toLowerCase();
+      key.shift = true;
 
-  } else if (parts = functionKeyCodeRe.exec(s)) {
-    // ansi escape sequence
+    } else if (parts = metaKeyCodeRe.exec(s)) {
+      // meta+character key
+      key.name = parts[1].toLowerCase();
+      key.meta = true;
+      key.shift = /^[A-Z]$/.test(parts[1]);
 
-    // reassemble the key code leaving out leading \x1b's,
-    // the modifier key bitflag and any meaningless "1;" sequence
-    var code = (parts[1] || '') + (parts[2] || '') +
-               (parts[4] || '') + (parts[6] || ''),
-        modifier = (parts[3] || parts[5] || 1) - 1;
+    } else if (parts = functionKeyCodeRe.exec(s)) {
+      // ansi escape sequence
 
-    // Parse the key modifier
-    key.ctrl = !!(modifier & 4);
-    key.meta = !!(modifier & 10);
-    key.shift = !!(modifier & 1);
-    key.code = code;
+      // reassemble the key code leaving out leading \x1b's,
+      // the modifier key bitflag and any meaningless "1;" sequence
+      var code = (parts[1] || '') + (parts[2] || '') +
+                 (parts[4] || '') + (parts[9] || ''),
+          modifier = (parts[3] || parts[8] || 1) - 1;
 
-    // Parse the key itself
-    switch (code) {
-      /* xterm/gnome ESC O letter */
-      case 'OP': key.name = 'f1'; break;
-      case 'OQ': key.name = 'f2'; break;
-      case 'OR': key.name = 'f3'; break;
-      case 'OS': key.name = 'f4'; break;
+      // Parse the key modifier
+      key.ctrl = !!(modifier & 4);
+      key.meta = !!(modifier & 10);
+      key.shift = !!(modifier & 1);
+      key.code = code;
 
-      /* xterm/rxvt ESC [ number ~ */
-      case '[11~': key.name = 'f1'; break;
-      case '[12~': key.name = 'f2'; break;
-      case '[13~': key.name = 'f3'; break;
-      case '[14~': key.name = 'f4'; break;
+      // Parse the key itself
+      switch (code) {
+        /* xterm/gnome ESC O letter */
+        case 'OP': key.name = 'f1'; break;
+        case 'OQ': key.name = 'f2'; break;
+        case 'OR': key.name = 'f3'; break;
+        case 'OS': key.name = 'f4'; break;
 
-      /* from Cygwin and used in libuv */
-      case '[[A': key.name = 'f1'; break;
-      case '[[B': key.name = 'f2'; break;
-      case '[[C': key.name = 'f3'; break;
-      case '[[D': key.name = 'f4'; break;
-      case '[[E': key.name = 'f5'; break;
+        /* xterm/rxvt ESC [ number ~ */
+        case '[11~': key.name = 'f1'; break;
+        case '[12~': key.name = 'f2'; break;
+        case '[13~': key.name = 'f3'; break;
+        case '[14~': key.name = 'f4'; break;
 
-      /* common */
-      case '[15~': key.name = 'f5'; break;
-      case '[17~': key.name = 'f6'; break;
-      case '[18~': key.name = 'f7'; break;
-      case '[19~': key.name = 'f8'; break;
-      case '[20~': key.name = 'f9'; break;
-      case '[21~': key.name = 'f10'; break;
-      case '[23~': key.name = 'f11'; break;
-      case '[24~': key.name = 'f12'; break;
+        /* from Cygwin and used in libuv */
+        case '[[A': key.name = 'f1'; break;
+        case '[[B': key.name = 'f2'; break;
+        case '[[C': key.name = 'f3'; break;
+        case '[[D': key.name = 'f4'; break;
+        case '[[E': key.name = 'f5'; break;
 
-      /* xterm ESC [ letter */
-      case '[A': key.name = 'up'; break;
-      case '[B': key.name = 'down'; break;
-      case '[C': key.name = 'right'; break;
-      case '[D': key.name = 'left'; break;
-      case '[E': key.name = 'clear'; break;
-      case '[F': key.name = 'end'; break;
-      case '[H': key.name = 'home'; break;
+        /* common */
+        case '[15~': key.name = 'f5'; break;
+        case '[17~': key.name = 'f6'; break;
+        case '[18~': key.name = 'f7'; break;
+        case '[19~': key.name = 'f8'; break;
+        case '[20~': key.name = 'f9'; break;
+        case '[21~': key.name = 'f10'; break;
+        case '[23~': key.name = 'f11'; break;
+        case '[24~': key.name = 'f12'; break;
 
-      /* xterm/gnome ESC O letter */
-      case 'OA': key.name = 'up'; break;
-      case 'OB': key.name = 'down'; break;
-      case 'OC': key.name = 'right'; break;
-      case 'OD': key.name = 'left'; break;
-      case 'OE': key.name = 'clear'; break;
-      case 'OF': key.name = 'end'; break;
-      case 'OH': key.name = 'home'; break;
+        /* xterm ESC [ letter */
+        case '[A': key.name = 'up'; break;
+        case '[B': key.name = 'down'; break;
+        case '[C': key.name = 'right'; break;
+        case '[D': key.name = 'left'; break;
+        case '[E': key.name = 'clear'; break;
+        case '[F': key.name = 'end'; break;
+        case '[H': key.name = 'home'; break;
 
-      /* xterm/rxvt ESC [ number ~ */
-      case '[1~': key.name = 'home'; break;
-      case '[2~': key.name = 'insert'; break;
-      case '[3~': key.name = 'delete'; break;
-      case '[4~': key.name = 'end'; break;
-      case '[5~': key.name = 'pageup'; break;
-      case '[6~': key.name = 'pagedown'; break;
+        /* xterm/gnome ESC O letter */
+        case 'OA': key.name = 'up'; break;
+        case 'OB': key.name = 'down'; break;
+        case 'OC': key.name = 'right'; break;
+        case 'OD': key.name = 'left'; break;
+        case 'OE': key.name = 'clear'; break;
+        case 'OF': key.name = 'end'; break;
+        case 'OH': key.name = 'home'; break;
 
-      /* putty */
-      case '[[5~': key.name = 'pageup'; break;
-      case '[[6~': key.name = 'pagedown'; break;
+        /* xterm/rxvt ESC [ number ~ */
+        case '[1~': key.name = 'home'; break;
+        case '[2~': key.name = 'insert'; break;
+        case '[3~': key.name = 'delete'; break;
+        case '[4~': key.name = 'end'; break;
+        case '[5~': key.name = 'pageup'; break;
+        case '[6~': key.name = 'pagedown'; break;
 
-      /* rxvt */
-      case '[7~': key.name = 'home'; break;
-      case '[8~': key.name = 'end'; break;
+        /* putty */
+        case '[[5~': key.name = 'pageup'; break;
+        case '[[6~': key.name = 'pagedown'; break;
 
-      /* rxvt keys with modifiers */
-      case '[a': key.name = 'up'; key.shift = true; break;
-      case '[b': key.name = 'down'; key.shift = true; break;
-      case '[c': key.name = 'right'; key.shift = true; break;
-      case '[d': key.name = 'left'; key.shift = true; break;
-      case '[e': key.name = 'clear'; key.shift = true; break;
+        /* rxvt */
+        case '[7~': key.name = 'home'; break;
+        case '[8~': key.name = 'end'; break;
 
-      case '[2$': key.name = 'insert'; key.shift = true; break;
-      case '[3$': key.name = 'delete'; key.shift = true; break;
-      case '[5$': key.name = 'pageup'; key.shift = true; break;
-      case '[6$': key.name = 'pagedown'; key.shift = true; break;
-      case '[7$': key.name = 'home'; key.shift = true; break;
-      case '[8$': key.name = 'end'; key.shift = true; break;
+        /* rxvt keys with modifiers */
+        case '[a': key.name = 'up'; key.shift = true; break;
+        case '[b': key.name = 'down'; key.shift = true; break;
+        case '[c': key.name = 'right'; key.shift = true; break;
+        case '[d': key.name = 'left'; key.shift = true; break;
+        case '[e': key.name = 'clear'; key.shift = true; break;
 
-      case 'Oa': key.name = 'up'; key.ctrl = true; break;
-      case 'Ob': key.name = 'down'; key.ctrl = true; break;
-      case 'Oc': key.name = 'right'; key.ctrl = true; break;
-      case 'Od': key.name = 'left'; key.ctrl = true; break;
-      case 'Oe': key.name = 'clear'; key.ctrl = true; break;
+        case '[2$': key.name = 'insert'; key.shift = true; break;
+        case '[3$': key.name = 'delete'; key.shift = true; break;
+        case '[5$': key.name = 'pageup'; key.shift = true; break;
+        case '[6$': key.name = 'pagedown'; key.shift = true; break;
+        case '[7$': key.name = 'home'; key.shift = true; break;
+        case '[8$': key.name = 'end'; key.shift = true; break;
 
-      case '[2^': key.name = 'insert'; key.ctrl = true; break;
-      case '[3^': key.name = 'delete'; key.ctrl = true; break;
-      case '[5^': key.name = 'pageup'; key.ctrl = true; break;
-      case '[6^': key.name = 'pagedown'; key.ctrl = true; break;
-      case '[7^': key.name = 'home'; key.ctrl = true; break;
-      case '[8^': key.name = 'end'; key.ctrl = true; break;
+        case 'Oa': key.name = 'up'; key.ctrl = true; break;
+        case 'Ob': key.name = 'down'; key.ctrl = true; break;
+        case 'Oc': key.name = 'right'; key.ctrl = true; break;
+        case 'Od': key.name = 'left'; key.ctrl = true; break;
+        case 'Oe': key.name = 'clear'; key.ctrl = true; break;
 
-      /* misc. */
-      case '[Z': key.name = 'tab'; key.shift = true; break;
-      default: key.name = 'undefined'; break;
+        case '[2^': key.name = 'insert'; key.ctrl = true; break;
+        case '[3^': key.name = 'delete'; key.ctrl = true; break;
+        case '[5^': key.name = 'pageup'; key.ctrl = true; break;
+        case '[6^': key.name = 'pagedown'; key.ctrl = true; break;
+        case '[7^': key.name = 'home'; key.ctrl = true; break;
+        case '[8^': key.name = 'end'; key.ctrl = true; break;
 
+        /* misc. */
+        case '[Z': key.name = 'tab'; key.shift = true; break;
+        default: key.name = 'undefined'; break;
+
+      }
     }
-  } else if (s.length > 1 && s[0] !== '\x1b') {
-    // Got a longer-than-one string of characters.
-    // Probably a paste, since it wasn't a control sequence.
-    Array.prototype.forEach.call(s, function(c) {
-      emitKey(stream, c);
-    });
-    return;
-  }
 
-  // Don't emit a key if no name was found
-  if (key.name === undefined) {
-    key = undefined;
-  }
+    // Don't emit a key if no name was found
+    if (util.isUndefined(key.name)) {
+      key = undefined;
+    }
 
-  if (s.length === 1) {
-    ch = s;
-  }
+    if (s.length === 1) {
+      ch = s;
+    }
 
-  if (key || ch) {
-    stream.emit('keypress', ch, key);
-  }
+    if (key || ch) {
+      stream.emit('keypress', ch, key);
+    }
+  });
 }
 
 
@@ -1096,13 +1183,16 @@ function emitKey(stream, s) {
  */
 
 function cursorTo(stream, x, y) {
-  if (typeof x !== 'number' && typeof y !== 'number')
+  if (util.isNullOrUndefined(stream))
     return;
 
-  if (typeof x !== 'number')
+  if (!util.isNumber(x) && !util.isNumber(y))
+    return;
+
+  if (!util.isNumber(x))
     throw new Error("Can't set cursor row without also setting it's column");
 
-  if (typeof y !== 'number') {
+  if (!util.isNumber(y)) {
     stream.write('\x1b[' + (x + 1) + 'G');
   } else {
     stream.write('\x1b[' + (y + 1) + ';' + (x + 1) + 'H');
@@ -1116,6 +1206,9 @@ exports.cursorTo = cursorTo;
  */
 
 function moveCursor(stream, dx, dy) {
+  if (util.isNullOrUndefined(stream))
+    return;
+
   if (dx < 0) {
     stream.write('\x1b[' + (-dx) + 'D');
   } else if (dx > 0) {
@@ -1139,6 +1232,9 @@ exports.moveCursor = moveCursor;
  */
 
 function clearLine(stream, dir) {
+  if (util.isNullOrUndefined(stream))
+    return;
+
   if (dir < 0) {
     // to the beginning
     stream.write('\x1b[1K');
@@ -1158,6 +1254,111 @@ exports.clearLine = clearLine;
  */
 
 function clearScreenDown(stream) {
+  if (util.isNullOrUndefined(stream))
+    return;
+
   stream.write('\x1b[0J');
 }
 exports.clearScreenDown = clearScreenDown;
+
+
+/**
+ * Returns the number of columns required to display the given string.
+ */
+
+function getStringWidth(str) {
+  var width = 0;
+  str = stripVTControlCharacters(str);
+  for (var i = 0, len = str.length; i < len; i++) {
+    var code = codePointAt(str, i);
+    if (code >= 0x10000) { // surrogates
+      i++;
+    }
+    if (isFullWidthCodePoint(code)) {
+      width += 2;
+    } else {
+      width++;
+    }
+  }
+  return width;
+}
+exports.getStringWidth = getStringWidth;
+
+
+/**
+ * Returns true if the character represented by a given
+ * Unicode code point is full-width. Otherwise returns false.
+ */
+
+function isFullWidthCodePoint(code) {
+  if (isNaN(code)) {
+    return false;
+  }
+
+  // Code points are derived from:
+  // http://www.unicode.org/Public/UNIDATA/EastAsianWidth.txt
+  if (code >= 0x1100 && (
+      code <= 0x115f ||  // Hangul Jamo
+      0x2329 === code || // LEFT-POINTING ANGLE BRACKET
+      0x232a === code || // RIGHT-POINTING ANGLE BRACKET
+      // CJK Radicals Supplement .. Enclosed CJK Letters and Months
+      (0x2e80 <= code && code <= 0x3247 && code !== 0x303f) ||
+      // Enclosed CJK Letters and Months .. CJK Unified Ideographs Extension A
+      0x3250 <= code && code <= 0x4dbf ||
+      // CJK Unified Ideographs .. Yi Radicals
+      0x4e00 <= code && code <= 0xa4c6 ||
+      // Hangul Jamo Extended-A
+      0xa960 <= code && code <= 0xa97c ||
+      // Hangul Syllables
+      0xac00 <= code && code <= 0xd7a3 ||
+      // CJK Compatibility Ideographs
+      0xf900 <= code && code <= 0xfaff ||
+      // Vertical Forms
+      0xfe10 <= code && code <= 0xfe19 ||
+      // CJK Compatibility Forms .. Small Form Variants
+      0xfe30 <= code && code <= 0xfe6b ||
+      // Halfwidth and Fullwidth Forms
+      0xff01 <= code && code <= 0xff60 ||
+      0xffe0 <= code && code <= 0xffe6 ||
+      // Kana Supplement
+      0x1b000 <= code && code <= 0x1b001 ||
+      // Enclosed Ideographic Supplement
+      0x1f200 <= code && code <= 0x1f251 ||
+      // CJK Unified Ideographs Extension B .. Tertiary Ideographic Plane
+      0x20000 <= code && code <= 0x3fffd)) {
+    return true;
+  }
+  return false;
+}
+exports.isFullWidthCodePoint = isFullWidthCodePoint;
+
+
+/**
+ * Returns the Unicode code point for the character at the
+ * given index in the given string. Similar to String.charCodeAt(),
+ * but this function handles surrogates (code point >= 0x10000).
+ */
+
+function codePointAt(str, index) {
+  var code = str.charCodeAt(index);
+  var low;
+  if (0xd800 <= code && code <= 0xdbff) { // High surrogate
+    low = str.charCodeAt(index + 1);
+    if (!isNaN(low)) {
+      code = 0x10000 + (code - 0xd800) * 0x400 + (low - 0xdc00);
+    }
+  }
+  return code;
+}
+exports.codePointAt = codePointAt;
+
+
+/**
+ * Tries to remove all VT control characters. Use to estimate displayed
+ * string width. May be buggy due to not running a real state machine
+ */
+function stripVTControlCharacters(str) {
+  str = str.replace(new RegExp(functionKeyCodeReAnywhere.source, 'g'), '');
+  return str.replace(new RegExp(metaKeyCodeReAnywhere.source, 'g'), '');
+}
+exports.stripVTControlCharacters = stripVTControlCharacters;
