@@ -949,7 +949,7 @@ REPLServer.prototype.convertToContext = function(cmd) {
 function isRecoverableError(e) {
   return e &&
       e.name === 'SyntaxError' &&
-      /^(Unexpected end of input|Unexpected token :)/.test(e.message);
+      /^(Unexpected end of input|Unexpected token)/.test(e.message);
 }
 
 function Recoverable(err) {
