@@ -53,7 +53,12 @@
     startup.processKillAndExit();
     startup.processSignalHandlers();
 
-    startup.processChannel();
+    // Do not initialize channel in debugger agent, it deletes env variable
+    // and the main thread won't see it.
+    if (process.argv[1] !== '--debug-agent')
+      startup.processChannel();
+
+    startup.processRawDebug();
 
     startup.resolveArgv0();
 
@@ -75,6 +80,11 @@
       var d = NativeModule.require('_debugger');
       d.start();
 
+    } else if (process.argv[1] == '--debug-agent') {
+      // Start the debugger agent
+      var d = NativeModule.require('_debugger_agent');
+      d.start();
+
     } else if (process._eval != null) {
       // User passed '-e' or '--eval' arguments to Node.
       evalScript('[eval]');
@@ -83,7 +93,7 @@
       var path = NativeModule.require('path');
       process.argv[1] = path.resolve(process.argv[1]);
 
-      // If this is a worker in cluster mode, start up the communiction
+      // If this is a worker in cluster mode, start up the communication
       // channel.
       if (process.env.NODE_UNIQUE_ID) {
         var cluster = NativeModule.require('cluster');
@@ -163,7 +173,6 @@
     global.GLOBAL = global;
     global.root = global;
     global.Buffer = NativeModule.require('buffer').Buffer;
-    process.binding('buffer').setFastBufferConstructor(global.Buffer);
     process.domain = null;
     process._exiting = false;
   };
@@ -217,61 +226,16 @@
   };
 
   startup.processFatal = function() {
-    // call into the active domain, or emit uncaughtException,
-    // and exit if there are no listeners.
     process._fatalException = function(er) {
-      var caught = false;
-      if (process.domain) {
-        var domain = process.domain;
-        var domainModule = NativeModule.require('domain');
-        var domainStack = domainModule._stack;
+      var caught;
 
-        // ignore errors on disposed domains.
-        //
-        // XXX This is a bit stupid.  We should probably get rid of
-        // domain.dispose() altogether.  It's almost always a terrible
-        // idea.  --isaacs
-        if (domain._disposed)
-          return true;
+      if (process.domain && process.domain._errorHandler)
+        caught = process.domain._errorHandler(er) || caught;
 
-        er.domain = domain;
-        er.domainThrown = true;
-        // wrap this in a try/catch so we don't get infinite throwing
-        try {
-          // One of three things will happen here.
-          //
-          // 1. There is a handler, caught = true
-          // 2. There is no handler, caught = false
-          // 3. It throws, caught = false
-          //
-          // If caught is false after this, then there's no need to exit()
-          // the domain, because we're going to crash the process anyway.
-          caught = domain.emit('error', er);
-
-          // Exit all domains on the stack.  Uncaught exceptions end the
-          // current tick and no domains should be left on the stack
-          // between ticks.
-          var domainModule = NativeModule.require('domain');
-          domainStack.length = 0;
-          domainModule.active = process.domain = null;
-        } catch (er2) {
-          // The domain error handler threw!  oh no!
-          // See if another domain can catch THIS error,
-          // or else crash on the original one.
-          // If the user already exited it, then don't double-exit.
-          if (domain === domainModule.active)
-            domainStack.pop();
-          if (domainStack.length) {
-            var parentDomain = domainStack[domainStack.length - 1];
-            process.domain = domainModule.active = parentDomain;
-            caught = process._fatalException(er2);
-          } else
-            caught = false;
-        }
-      } else {
+      if (!caught)
         caught = process.emit('uncaughtException', er);
-      }
-      // if someone handled it, then great.  otherwise, die in C++ land
+
+      // If someone handled it, then great.  otherwise, die in C++ land
       // since that means that we'll exit the process, emit the 'exit' event
       if (!caught) {
         try {
@@ -282,19 +246,18 @@
         } catch (er) {
           // nothing to be done about it at this point.
         }
-      }
+
       // if we handled an error, then make sure any ticks get processed
-      if (caught)
-        process._needTickCallback();
+      } else {
+        NativeModule.require('timers').setImmediate(process._tickCallback);
+      }
+
       return caught;
     };
   };
 
   var assert;
   startup.processAssert = function() {
-    // Note that calls to assert() are pre-processed out by JS2C for the
-    // normal build of node. They persist only in the node_g build.
-    // Similarly for debug().
     assert = process.assert = function(x, msg) {
       if (!x) throw new Error(msg || 'assertion error');
     };
@@ -306,7 +269,11 @@
     delete NativeModule._source.config;
 
     // strip the gyp comment line at the beginning
-    config = config.split('\n').slice(1).join('\n').replace(/'/g, '"');
+    config = config.split('\n')
+                   .slice(1)
+                   .join('\n')
+                   .replace(/"/g, '\\"')
+                   .replace(/'/g, '"');
 
     process.config = JSON.parse(config, function(key, value) {
       if (value === 'true') return true;
@@ -316,196 +283,128 @@
   };
 
   startup.processNextTick = function() {
-    var _needTickCallback = process._needTickCallback;
     var nextTickQueue = [];
-    var needSpinner = true;
-    var inTick = false;
+    var microtasksScheduled = false;
 
-    // this infobox thing is used so that the C++ code in src/node.cc
+    // Used to run V8's micro task queue.
+    var _runMicrotasks = {};
+
+    // This tickInfo thing is used so that the C++ code in src/node.cc
     // can have easy accesss to our nextTick state, and avoid unnecessary
-    // calls into process._tickCallback.
-    // order is [length, index, depth]
-    // Never write code like this without very good reason!
-    var infoBox = process._tickInfoBox;
-    var length = 0;
-    var index = 1;
-    var depth = 2;
+    var tickInfo = {};
+
+    // *Must* match Environment::TickInfo::Fields in src/env.h.
+    var kIndex = 0;
+    var kLength = 1;
 
     process.nextTick = nextTick;
-    // needs to be accessible from cc land
-    process._nextDomainTick = _nextDomainTick;
+    // Needs to be accessible from beyond this scope.
     process._tickCallback = _tickCallback;
     process._tickDomainCallback = _tickDomainCallback;
-    process._tickFromSpinner = _tickFromSpinner;
 
-    // the maximum number of times it'll process something like
-    // nextTick(function f(){nextTick(f)})
-    // It's unlikely, but not illegal, to hit this limit.  When
-    // that happens, it yields to libuv's tick spinner.
-    // This is a loop counter, not a stack depth, so we aren't using
-    // up lots of memory here.  I/O can sneak in before nextTick if this
-    // limit is hit, which is not ideal, but not terrible.
-    process.maxTickDepth = 1000;
+    process._setupNextTick(tickInfo, _tickCallback, _runMicrotasks);
 
-    function tickDone(tickDepth_) {
-      if (infoBox[length] !== 0) {
-        if (infoBox[length] <= infoBox[index]) {
+    _runMicrotasks = _runMicrotasks.runMicrotasks;
+
+    function tickDone() {
+      if (tickInfo[kLength] !== 0) {
+        if (tickInfo[kLength] <= tickInfo[kIndex]) {
           nextTickQueue = [];
-          infoBox[length] = 0;
+          tickInfo[kLength] = 0;
         } else {
-          nextTickQueue.splice(0, infoBox[index]);
-          infoBox[length] = nextTickQueue.length;
-          if (needSpinner) {
-            _needTickCallback();
-            needSpinner = false;
-          }
+          nextTickQueue.splice(0, tickInfo[kIndex]);
+          tickInfo[kLength] = nextTickQueue.length;
         }
       }
-      inTick = false;
-      infoBox[index] = 0;
-      infoBox[depth] = tickDepth_;
+      tickInfo[kIndex] = 0;
     }
 
-    function maxTickWarn() {
-      // XXX Remove all this maxTickDepth stuff in 0.11
-      var msg = '(node) warning: Recursive process.nextTick detected. ' +
-                'This will break in the next version of node. ' +
-                'Please use setImmediate for recursive deferral.';
-      if (process.throwDeprecation)
-        throw new Error(msg);
-      else if (process.traceDeprecation)
-        console.trace(msg);
-      else
-        console.error(msg);
-    }
-
-    function _tickFromSpinner() {
-      needSpinner = true;
-      // coming from spinner, reset!
-      if (infoBox[depth] !== 0)
-        infoBox[depth] = 0;
-      // no callbacks to run
-      if (infoBox[length] === 0)
-        return infoBox[index] = infoBox[depth] = 0;
-      process._tickCallback();
-    }
-
-    // run callbacks that have no domain
-    // using domains will cause this to be overridden
-    function _tickCallback() {
-      var callback, nextTickLength, threw;
-
-      if (inTick) return;
-      if (infoBox[length] === 0) {
-        infoBox[index] = 0;
-        infoBox[depth] = 0;
+    function scheduleMicrotasks() {
+      if (microtasksScheduled)
         return;
-      }
-      inTick = true;
 
-      while (infoBox[depth]++ < process.maxTickDepth) {
-        nextTickLength = infoBox[length];
-        if (infoBox[index] === nextTickLength)
-          return tickDone(0);
+      nextTickQueue.push({
+        callback: runMicrotasksCallback,
+        domain: null
+      });
 
-        while (infoBox[index] < nextTickLength) {
-          callback = nextTickQueue[infoBox[index]++].callback;
-          threw = true;
-          try {
-            callback();
-            threw = false;
-          } finally {
-            if (threw) tickDone(infoBox[depth]);
-          }
+      tickInfo[kLength]++;
+      microtasksScheduled = true;
+    }
+
+    function runMicrotasksCallback() {
+      microtasksScheduled = false;
+      _runMicrotasks();
+
+      if (tickInfo[kIndex] < tickInfo[kLength])
+        scheduleMicrotasks();
+    }
+
+    // Run callbacks that have no domain.
+    // Using domains will cause this to be overridden.
+    function _tickCallback() {
+      var callback, threw, tock;
+
+      scheduleMicrotasks();
+
+      while (tickInfo[kIndex] < tickInfo[kLength]) {
+        tock = nextTickQueue[tickInfo[kIndex]++];
+        callback = tock.callback;
+        threw = true;
+        try {
+          callback();
+          threw = false;
+        } finally {
+          if (threw)
+            tickDone();
         }
+        if (1e4 < tickInfo[kIndex])
+          tickDone();
       }
 
-      tickDone(0);
+      tickDone();
     }
 
     function _tickDomainCallback() {
-      var nextTickLength, tock, callback, threw;
+      var callback, domain, threw, tock;
 
-      // if you add a nextTick in a domain's error handler, then
-      // it's possible to cycle indefinitely.  Normally, the tickDone
-      // in the finally{} block below will prevent this, however if
-      // that error handler ALSO triggers multiple MakeCallbacks, then
-      // it'll try to keep clearing the queue, since the finally block
-      // fires *before* the error hits the top level and is handled.
-      if (infoBox[depth] >= process.maxTickDepth)
-        return _needTickCallback();
+      scheduleMicrotasks();
 
-      if (inTick) return;
-      inTick = true;
-
-      // always do this at least once.  otherwise if process.maxTickDepth
-      // is set to some negative value, or if there were repeated errors
-      // preventing depth from being cleared, we'd never process any
-      // of them.
-      while (infoBox[depth]++ < process.maxTickDepth) {
-        nextTickLength = infoBox[length];
-        if (infoBox[index] === nextTickLength)
-          return tickDone(0);
-
-        while (infoBox[index] < nextTickLength) {
-          tock = nextTickQueue[infoBox[index]++];
-          callback = tock.callback;
-          if (tock.domain) {
-            if (tock.domain._disposed) continue;
-            tock.domain.enter();
-          }
-          threw = true;
-          try {
-            callback();
-            threw = false;
-          } finally {
-            // finally blocks fire before the error hits the top level,
-            // so we can't clear the depth at this point.
-            if (threw) tickDone(infoBox[depth]);
-          }
-          if (tock.domain) {
-            tock.domain.exit();
-          }
+      while (tickInfo[kIndex] < tickInfo[kLength]) {
+        tock = nextTickQueue[tickInfo[kIndex]++];
+        callback = tock.callback;
+        domain = tock.domain;
+        if (domain)
+          domain.enter();
+        threw = true;
+        try {
+          callback();
+          threw = false;
+        } finally {
+          if (threw)
+            tickDone();
         }
+        if (1e4 < tickInfo[kIndex])
+          tickDone();
+        if (domain)
+          domain.exit();
       }
 
-      tickDone(0);
+      tickDone();
     }
 
     function nextTick(callback) {
       // on the way out, don't bother. it won't get fired anyway.
       if (process._exiting)
         return;
-      if (infoBox[depth] >= process.maxTickDepth)
-        maxTickWarn();
 
-      var obj = { callback: callback, domain: null };
+      var obj = {
+        callback: callback,
+        domain: process.domain || null
+      };
 
       nextTickQueue.push(obj);
-      infoBox[length]++;
-
-      if (needSpinner) {
-        _needTickCallback();
-        needSpinner = false;
-      }
-    }
-
-    function _nextDomainTick(callback) {
-      // on the way out, don't bother. it won't get fired anyway.
-      if (process._exiting)
-        return;
-      if (infoBox[depth] >= process.maxTickDepth)
-        maxTickWarn();
-
-      var obj = { callback: callback, domain: process.domain };
-
-      nextTickQueue.push(obj);
-      infoBox[length]++;
-
-      if (needSpinner) {
-        _needTickCallback();
-        needSpinner = false;
-      }
+      tickInfo[kLength]++;
     }
   };
 
@@ -526,23 +425,13 @@
                'global.__dirname = __dirname;\n' +
                'global.require = require;\n' +
                'return require("vm").runInThisContext(' +
-               JSON.stringify(body) + ', ' +
-               JSON.stringify(name) + ', true);\n';
+               JSON.stringify(body) + ', { filename: ' +
+               JSON.stringify(name) + ' });\n';
     }
     var result = module._compile(script, name + '-wrapper');
     if (process._print_eval) console.log(result);
   }
 
-  function errnoException(errorno, syscall) {
-    // TODO make this more compatible with ErrnoException from src/node.cc
-    // Once all of Node is using this function the ErrnoException from
-    // src/node.cc should be removed.
-    var e = new Error(syscall + ' ' + errorno);
-    e.errno = e.code = errorno;
-    e.syscall = syscall;
-    return e;
-  }
-
   function createWritableStdioStream(fd) {
     var stream;
     var tty_wrap = process.binding('tty_wrap');
@@ -564,7 +453,7 @@
 
       case 'FILE':
         var fs = NativeModule.require('fs');
-        stream = new fs.SyncWriteStream(fd);
+        stream = new fs.SyncWriteStream(fd, { autoClose: false });
         stream._type = 'fs';
         break;
 
@@ -651,7 +540,7 @@
 
         case 'FILE':
           var fs = NativeModule.require('fs');
-          stdin = new fs.ReadStream(null, { fd: fd });
+          stdin = new fs.ReadStream(null, { fd: fd, autoClose: false });
           break;
 
         case 'PIPE':
@@ -701,31 +590,41 @@
   };
 
   startup.processKillAndExit = function() {
+
     process.exit = function(code) {
+      if (code || code === 0)
+        process.exitCode = code;
+
       if (!process._exiting) {
         process._exiting = true;
-        process.emit('exit', code || 0);
+        process.emit('exit', process.exitCode || 0);
       }
-      process.reallyExit(code || 0);
+      process.reallyExit(process.exitCode || 0);
     };
 
     process.kill = function(pid, sig) {
-      var r;
+      var err;
+
+      if (pid != (pid | 0)) {
+        throw new TypeError('invalid pid');
+      }
 
       // preserve null signal
       if (0 === sig) {
-        r = process._kill(pid, 0);
+        err = process._kill(pid, 0);
       } else {
         sig = sig || 'SIGTERM';
-        if (startup.lazyConstants()[sig]) {
-          r = process._kill(pid, startup.lazyConstants()[sig]);
+        if (startup.lazyConstants()[sig] &&
+            sig.slice(0, 3) === 'SIG') {
+          err = process._kill(pid, startup.lazyConstants()[sig]);
         } else {
           throw new Error('Unknown signal: ' + sig);
         }
       }
 
-      if (r) {
-        throw errnoException(process._errno, 'kill');
+      if (err) {
+        var errnoException = NativeModule.require('util')._errnoException;
+        throw errnoException(err, 'kill');
       }
 
       return true;
@@ -756,10 +655,11 @@
         wrap.onsignal = function() { process.emit(type); };
 
         var signum = startup.lazyConstants()[type];
-        var r = wrap.start(signum);
-        if (r) {
+        var err = wrap.start(signum);
+        if (err) {
           wrap.close();
-          throw errnoException(process._errno, 'uv_signal_start');
+          var errnoException = NativeModule.require('util')._errnoException;
+          throw errnoException(err, 'uv_signal_start');
         }
 
         signalWraps[type] = wrap;
@@ -773,7 +673,7 @@
       if (isSignal(type)) {
         assert(signalWraps.hasOwnProperty(type));
 
-        if (this.listeners(type).length === 0) {
+        if (NativeModule.require('events').listenerCount(this, type) === 0) {
           signalWraps[type].close();
           delete signalWraps[type];
         }
@@ -804,7 +704,17 @@
       cp._forkChild(fd);
       assert(process.send);
     }
-  }
+  };
+
+
+  startup.processRawDebug = function() {
+    var format = NativeModule.require('util').format;
+    var rawDebug = process._rawDebug;
+    process._rawDebug = function() {
+      rawDebug(format.apply(null, arguments));
+    };
+  };
+
 
   startup.resolveArgv0 = function() {
     var cwd = process.cwd();
@@ -826,8 +736,11 @@
   // core modules found in lib/*.js. All core modules are compiled into the
   // node binary, so they can be loaded faster.
 
-  var Script = process.binding('evals').NodeScript;
-  var runInThisContext = Script.runInThisContext;
+  var ContextifyScript = process.binding('contextify').ContextifyScript;
+  function runInThisContext(code, options) {
+    var script = new ContextifyScript(code, options);
+    return script.runInThisContext();
+  }
 
   function NativeModule(id) {
     this.filename = id + '.js';
@@ -888,7 +801,7 @@
     var source = NativeModule.getSource(this.id);
     source = NativeModule.wrap(source);
 
-    var fn = runInThisContext(source, this.filename, true);
+    var fn = runInThisContext(source, { filename: this.filename });
     fn(this.exports, NativeModule.require, this, this.filename);
 
     this.loaded = true;
